import requests
from requests.auth import HTTPBasicAuth
import logging
from datetime import datetime, date, timedelta
import json

today = date.today()
today_logs = today.strftime("%d_%m_%Y")

logging.basicConfig(filename=("logs/shoppster_orders_" + today_logs +".log"), level=logging.INFO, format='%(asctime)s %(levelname)s:%(message)s', datefmt='%Y-%m-%d %H:%M:%S')

auth=HTTPBasicAuth('shumee_integration_API', 'R!V)8$Y2zB3v')

def pretty_print_request(req):
    """
    At this point it is completely built and ready
    to be fired; it is "prepared".

    However pay attention at the formatting used in 
    this function because it is programmed to be pretty 
    printed and may differ from the actual request.
    """
    headers = '\r\n'.join('{}: {}'.format(k, v) for k, v in req.headers.items())
    print(f'REQUEST\n\nmethod: {req.method}\n\nurl: {req.url}\r\n\nheaders: { headers }\r\n\nbody: {req.body}\n')

def get_all_cons():
    print("get_all_cons")
    logging.info("get_all_cons")

    now = datetime.now()
    yesterday_midnight = (now - timedelta(days=1)).replace(hour=0, minute=0, second=0, microsecond=0)
    dateFrom = yesterday_midnight.isoformat(timespec='seconds')
    dateTo = now.isoformat(timespec='seconds')
    
    params = {
        "dateFrom": dateFrom,
        "dateTo": dateTo,
        "warehouseCode": "0178",
        "consignmentStatus": "SHIPPED_TO_CUSTOMER"
    }

    r = requests.get("https://stage-api.shoppster.com/rest_basic/v2/ung-site-rs/consignments", params = params, auth = auth)
    req = requests.Request("GET", "https://stage-api.shoppster.com/rest_basic/v2/ung-site-rs/consignments", params = params, auth = auth)
    req = req.prepare()
    pretty_print_request(req)
    print(f"RESPONSE\n\nstatus code: {r.status_code}\n\nresponse text: {json.dumps(r.json(), indent=2)}")
    logging.info(f"{r.status_code}\n{json.dumps(r.json(), indent=2)}")
    return r.json()
    #return [{"code":"S000004007209_0"},{"code":"S000004007211_0"},{"code":"S000004007213_0"}]

def get_cons_details(code):
    print("get_cons_details")
    logging.info("get_cons_details")
    params = {
        "code": code
    }
    r = requests.get("https://stage-api.shoppster.com/rest_basic/v2/ung-site-rs/consignmentByCode", params = params, auth = auth)
    req = requests.Request("GET", "https://stage-api.shoppster.com/rest_basic/v2/ung-site-rs/consignmentByCode", params = params, auth = auth)
    req = req.prepare()
    pretty_print_request(req)
    try:
        print(f"RESPONSE\n\nstatus code: {r.status_code}\n\nresponse text: {json.dumps(r.json(), indent=2)}")
        logging.info(f"status: {r.status_code}\nresponse: {json.dumps(r.json(), indent=2)}")
    except:
        print(f"RESPONSE\n\nstatus code: {r.status_code}\n\nresponse text: {r.text}")
        logging.info(f"status: {r.status_code}\nresponse: {r.text}")
    return r.json()
    
def post_cons_in_status_fulfillment(code):
    print("post_cons_in_status_fulfillment")
    logging.info("post_cons_in_status_fulfillment")
    #headers = 
    data = {
        "@odata.context": "$metadata#Consignments/$entity",
        "code": code,
        "status": {
            "code": "IN_FULFILLMENT"
        }
    }

    r = requests.post("https://stage-api.shoppster.com/odata2webservices/InboundExtDropshippingConsignment/Consignments", json = data, auth = auth)
    req = requests.Request("POST", "https://stage-api.shoppster.com/odata2webservices/InboundExtDropshippingConsignment/Consignments", json = data, auth = auth)
    req = req.prepare()
    pretty_print_request(req)
    try:
        print(f"RESPONSE\n\nstatus code: {r.status_code}\n\nresponse text: {json.dumps(r.json(), indent=2)}")
        logging.info(f"{r.status_code}\n{json.dumps(r.json(), indent=2)}")
    except:
        print(f"RESPONSE\n\nstatus code: {r.status_code}\n\nresponse text: {r.text}")
        logging.info(f"{r.status_code}\n{r.text}")

def cancel_order(code):
    print("post_cons_in_status_fulfillment")
    logging.info("post_cons_in_status_fulfillment")
    #headers = 
    data = {
        "@odata.context": "$metadata#Consignments/$entity",
        "code": code,
        "status": {
            "code": "CANCELED"
        }
    }

    r = requests.post("https://stage-api.shoppster.com/odata2webservices/InboundExtDropshippingConsignment/Consignments", json = data, auth = auth)
    req = requests.Request("POST", "https://stage-api.shoppster.com/odata2webservices/InboundExtDropshippingConsignment/Consignments", json = data, auth = auth)
    req = req.prepare()
    pretty_print_request(req)
    try:
        print(f"RESPONSE\n\nstatus code: {r.status_code}\n\nresponse text: {json.dumps(r.json(), indent=2)}")
        logging.info(f"{r.status_code}\n{json.dumps(r.json(), indent=2)}")
    except:
        print(f"RESPONSE\n\nstatus code: {r.status_code}\n\nresponse text: {r.text}")
        logging.info(f"{r.status_code}\n{r.text}")
        
def add_order(bl_body):
    print("add_order")
    logging.info("add_order")
    data = {
      'token': "3001458-3006626-L61DQOGCX4XJFS6KB8HY63VR8L8BS0LYK8ICVKNYYYXW5DQMJV164W0AT9KNZN3E",
      'method': 'addOrder',
      'parameters': json.dumps(bl_body)
      }
    r = requests.post("https://api.baselinker.com/connector.php", data=data)
    print(f"{r.status_code}\n{json.dumps(r.json(), indent=2)}")
    logging.info(f"{r.status_code}\n{json.dumps(r.json(), indent=2)}")
    
def send_order_to_baselinker(order):
    date_format = datetime.strptime(order['orderDate'], "%d.%m.%Y.")
    unix_time = datetime.timestamp(date_format)
    bl_body = {
        "order_status_id": 314830,
        "date_add": unix_time,
        "phone": order['deliveryAddress']['cellphone'],
        "email": order['deliveryAddress']['email'],
        "user_comments": '',
        "user_login": order['billingAddress']['email'],
        "admin_comments": "",
        "currency": "EUR",
        "paid": "1",
        "delivery_fullname": f"{order['deliveryAddress']['firstname']} {order['deliveryAddress']['lastname']}",
        "delivery_address": f"{order['deliveryAddress']['streetname']} {order['deliveryAddress']['streetnumber']}",
        "delivery_city": order['deliveryAddress']['town'],
        "delivery_postcode": order['deliveryAddress']['postalcode'],
        "delivery_country_code": order['deliveryAddress']['country'],
        "invoice_fullname": f"{order['billingAddress']['firstname']} {order['billingAddress']['lastname']}",
        "invoice_address": f"{order['billingAddress']['streetname']} {order['billingAddress']['streetnumber']}",
        "invoice_city": order['billingAddress']['town'],
        "invoice_postcode": order['billingAddress']['postalcode'],
        "invoice_country_code": order['billingAddress']['country'],
        "custom_source_id": 33466,
        "delivery_price": "0",#order['deliveryCost'],
        "custom_extra_fields": {"77449": order['code']},
        "products": []
    }
    for prod in order['consignmentEntries']:
        bl_product = {
            "storage": "db",
            "storage_id": 0,
            "product_id": prod['vendorSKU'],
            "name": prod['vendorSKU'],
            "sku": prod['vendorSKU'],
            "ean": prod['productEan'],
            "price_brutto": prod['productTotalPrice'],
            "tax_rate": 0,
            "quantity": prod['productQuantity'],
            }
        bl_body['products'].append(bl_product)
    try:
        bl_body["invoice_nip"] =  order['billingAddress']['vatNumber']
        bl_body["invoice_company"] = order['billingAddress']['company']
    except:
        pass
        
    add_order(bl_body)

if __name__ == "__main__":
    cons = get_all_cons()
    
    #codes = ['S000004100587_0', 'S000004100591_0', 'S000004100704_0']
    #for code in codes:
        #con_details = get_cons_details(code)
        #send_order_to_baselinker(con_details)
        #post_cons_in_status_fulfillment(code)
    
    #for con in cons:
        #con_details = get_cons_details(con['code'])
        #send_order_to_baselinker(con_details)
        #post_cons_in_status_fulfillment(con['code'])